"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __spreadArrays = (this && this.__spreadArrays) || function () {
    for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
    for (var r = Array(s), k = 0, i = 0; i < il; i++)
        for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
            r[k] = a[j];
    return r;
};
exports.__esModule = true;
exports.Logger = exports.toJson = exports.localISOdt = void 0;
var fs_1 = require("fs");
var local_iso_dt_1 = require("local-iso-dt");
exports.localISOdt = local_iso_dt_1.localISOdt;
var cleanStack = require("clean-stack");
var SE = require("serialize-error");
var stream_1 = require("stream");
function toJson(object, spaces) {
    if (spaces === void 0) { spaces = 2; }
    return JSON.stringify(object, null, spaces);
}
exports.toJson = toJson;
function recursivelyCleanStack(thing) {
    if (!thing || thing instanceof Date)
        return;
    if (thing === null || thing === void 0 ? void 0 : thing.stack)
        try {
            return thing.stack = cleanStack(thing.stack, { pretty: true });
        }
        catch (_a) {
            return 'thing.stack was read-only';
        }
    if (Array.isArray(thing))
        thing.forEach(function (e) { return recursivelyCleanStack(e); });
    else if (thing instanceof Set || thing instanceof Map)
        for (var _i = 0, _b = thing.values(); _i < _b.length; _i++) {
            var value = _b[_i];
            cleanStack(value);
        }
    else if (typeof thing === 'object')
        Object.values(thing).forEach(function (value) { return recursivelyCleanStack(value); });
}
var Logger = (function (_super) {
    __extends(Logger, _super);
    function Logger(filename) {
        var _this = _super.call(this) || this;
        _this.filename = filename;
        if (filename)
            _this.stream = fs_1.createWriteStream(filename, { flags: 'a' });
        return _this;
    }
    Logger.timestamp = function (datetime) {
        return "[" + local_iso_dt_1.localISOdt(datetime).replace('T', ' ') + "]";
    };
    Logger.message2json = function (potentialObject) {
        if (typeof potentialObject === 'object')
            return toJson(SE.serializeError(potentialObject), 4).replace(/\\n/g, '\n');
        if (typeof potentialObject === 'string')
            return potentialObject.replace(/\\n/g, '\n');
        return potentialObject;
    };
    Logger.messages2json = function (messages) {
        return messages.map(Logger.message2json).join(' ').replace(/\u001b\[\d\w/g, '');
    };
    Logger.prepareMessages = function (messages) {
        recursivelyCleanStack(messages);
        var initialNewlines = '';
        try {
            initialNewlines = messages[0].match(/^\n+/)[0];
            messages[0] = messages[0].slice(initialNewlines.length);
        }
        catch (_a) { }
        return initialNewlines + Logger.timestamp();
    };
    Logger.prototype.debug = function () {
        var _a;
        var messages = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            messages[_i] = arguments[_i];
        }
        var prefix = Logger.prepareMessages(messages);
        console.debug.apply(console, __spreadArrays(["\u001B[90m" + prefix + "\u001B[39m"], messages));
        (_a = this.stream) === null || _a === void 0 ? void 0 : _a.write(prefix + " debug: " + Logger.messages2json(messages) + "\n");
    };
    Logger.prototype.info = function () {
        var _a;
        var messages = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            messages[_i] = arguments[_i];
        }
        var prefix = Logger.prepareMessages(messages);
        console.info.apply(console, __spreadArrays([prefix], messages));
        (_a = this.stream) === null || _a === void 0 ? void 0 : _a.write(prefix + " info: " + Logger.messages2json(messages) + "\n");
    };
    Logger.prototype.warn = function () {
        var _a;
        var messages = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            messages[_i] = arguments[_i];
        }
        var prefix = Logger.prepareMessages(messages);
        console.warn.apply(console, __spreadArrays(["\u001B[33m" + prefix + " WARN:\u001B[39m"], messages));
        (_a = this.stream) === null || _a === void 0 ? void 0 : _a.write(prefix + " WARN: " + Logger.messages2json(messages) + "\n");
    };
    Logger.prototype.error = function () {
        var _a;
        var messages = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            messages[_i] = arguments[_i];
        }
        var prefix = Logger.prepareMessages(messages);
        console.error.apply(console, __spreadArrays(["\u001B[31m" + prefix + " ERROR:\u001B[39m"], messages));
        (_a = this.stream) === null || _a === void 0 ? void 0 : _a.write(prefix + " ERROR: " + Logger.messages2json(messages) + "\n");
    };
    Logger.prototype.write = function (message) {
        var _a;
        var prefix = Logger.prepareMessages([message]);
        console.debug(prefix, message);
        (_a = this.stream) === null || _a === void 0 ? void 0 : _a.write(prefix + " debug: " + Logger.message2json(message) + "\n");
        return true;
    };
    Logger.prototype.teardown = function () {
        var _a;
        (_a = this.stream) === null || _a === void 0 ? void 0 : _a.end();
    };
    return Logger;
}(stream_1.Stream.Writable));
exports.Logger = Logger;
